package com.logicbig.example;

import javax.persistence.*;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            reset();
            persistEntities();
            findAllEmployeeEntities();
            movePersonToHistoryByName("Dana");
            movePersonToHistoryByName("Mike");
            fetchPersonHistory();

        } finally {
            entityManagerFactory.close();
        }
    }

    //calling procedure with ref_cursor
    private static void fetchPersonHistory() {
        System.out.println("-- Fetching person History --");
        EntityManager entityManager = entityManagerFactory.createEntityManager();
        StoredProcedureQuery procedureQuery =
                entityManager.createNamedStoredProcedureQuery(Person.NamedQuery_FetchFromHistory);
        procedureQuery.execute();
        @SuppressWarnings("unchecked")
        List<Person> resultList = procedureQuery.getResultList();
        resultList.forEach(System.out::println);
    }

    private static void movePersonToHistoryByName(String name) {
        System.out.printf("-- Moving person to history table name: %s --%n", name);
        EntityManager entityManager = entityManagerFactory.createEntityManager();
        //get person id
        TypedQuery<Long> query = entityManager
                .createQuery("SELECT p.id FROM Person p WHERE p.firstName = :firstName", Long.class);
        query.setParameter("firstName", name);
        Long personId = query.getSingleResult();
        //stored procedure
        StoredProcedureQuery procedureQuery = entityManager
                .createNamedStoredProcedureQuery(Person.NamedQuery_MoveToHistory);
        entityManager.getTransaction().begin();
        procedureQuery.setParameter("person_id_in", personId);
        procedureQuery.execute();
        Object msg_out = procedureQuery.getOutputParameterValue("msg_out");
        System.out.println("Out msg= " + msg_out);
        entityManager.getTransaction().commit();
    }

    private static void findAllEmployeeEntities() {
        System.out.println("-- all Person entities -  --");
        EntityManager em = entityManagerFactory.createEntityManager();
        TypedQuery<Person> query = em.createQuery("SELECT p from Person p", Person.class);
        List<Person> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void persistEntities() {
        Person person1 = Person.create("Dana", "Whitley", "464 Gorsuch Drive");
        Person person2 = Person.create("Robin", "Cash", "64 Zella Park");
        Person person3 = Person.create("Chary", "Mess", "112 Yellow Hill");
        Person person4 = Person.create("Rose", "Kantata", "2736 Kooter Lane");
        Person person5 = Person.create("Mike","Togglie", "111 Cool Dr");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(person1);
        em.persist(person2);
        em.persist(person3);
        em.persist(person4);
        em.persist(person5);
        em.getTransaction().commit();
        em.close();
    }

    private static void reset() {
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        Query query = em.createQuery("DELETE FROM Person");
        query.executeUpdate();
        query = em.createNativeQuery("DELETE from PERSON_HISTORY");
        query.executeUpdate();
        em.getTransaction().commit();
    }
}